tmepath = os.getenv("TME_PATH_1_3_0")
package.path = package.path .. ";"..tmepath.."/contrib/courselib/?.lua;./?.lua"


local ca     = require ("ca")
local laux   = require ("laux")

WAIT = 5
DEAD = 0
ALIVE = 1
 
wolfram = ca.Model {
    time = 1,
    rule_number = 110,    -- wolfram's rule number (0 to 255)
    cells = CellularSpace {
        xdim = 120, 
		ydim = 60,
        instance = { state = DEAD }
       },
    init = function (model)
        -- set up the rules for each configuration
        model.rules = laux.to8Bits (model.rule_number)
        -- put a single 1 in the mid point of the first line
        local mid = (model.cells.xdim + 1 )/2 -1
        model.cells:getCell(Coord{x=mid, y=0}).state = ALIVE
    end,
    
    changes = function (model)
        -- TerraME arrays start at 0, Lua's at 1 
        for i = 0, model.cells.xdim - 1 do
                -- coordinates of the left, centre and right points
                local leftx   = ((i - 1) > 0) and (i - 1) or (model.cells.xdim - 1)
                local centerx = i
                local rightx  = ((i + 1) == model.cells.xdim ) and 0 or (i + 1)
                local yc      = model.time - 1
                -- values in the three cells above the current one
                local topl = model.cells:getCell(Coord {x = leftx,   y = yc}).state
                local topc = model.cells:getCell(Coord {x = centerx, y = yc}).state
                local topr = model.cells:getCell(Coord {x = rightx,  y = yc}).state
                
                -- combine the values in a ky from "000" to "111" 
                local conf = ""..topl..topc..topr
                -- assign the values to the current cell
                local xy   = Coord {x = centerx,  y = yc + 1} 
                model.cells:getCell(xy).state = model.rules [conf]
        end
        laux.wait (WAIT)
    end,
    
    maps = { theme1 = {{attribute = "state", colors = "Blues", values = {DEAD, ALIVE}}}
           }
}

wolfram:run (59)

